﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Forms;
using System.Linq;
using System.Text;
using System.IO.Ports;
using System.Threading;
/*  This class provides for many of the usual features needed for communicating
 *  with the VTM series 3000 controllers. Highlights follow:
 *  
 *  StartCom()               Two overloaded types to begin serial communications with one for the
 *                               series 3000 and another for general sending/receiving.
 *  EndCom()                 Ends serial communications for any open comport.
 *  
 *  Parse()                  Return the next comma delimited string and delete it from the input string.
 *  CheckSum()               Return a one byte check sum for six bytes of data as required by the series 3000.
 *  ShowComm()               Use for debugging to return a string of the bytes sent or received
 *  
 *  FillComboBoxWithPorts()  Fill a combobox with the available comports.
 *  ChangeSerialPort()       Takes a new serial port as an argument and starts it for the series 3000.
 *  
 *  GetData()                Two overloaded versions to get data specifically from the series 3000.
 *  GetThree()               Tries to call getdata() three times before returning an error.
 *  Send()                   A generic send function just sends a string argument.
 *  SendLine()               This sends and receives entire lines by adding the CR,LF to the sent string and 
 *                               waiting for the return line. ReadTimeOut applies to this.
 *  SendBinary()             Sends a binary array of bytes.
 *  
 *  Read()                   Reads in a string that is terminated with a CR or LF.
 *  ReadBinary()             Reads in a binary array of bytes.
*/
namespace VtmStuff
{
    public class VtmComm
    {

        public static string Parse(ref String str)
        {
            String s = "";
            int i = 0;
            if (str.Length > 0)
            {
                while ((i < str.Length) && (str[i] != ','))
                {
                    s = s + str[i];
                    i++;
                }//while
                if (i == str.Length)
                {
                    str = "";
                }//last parser
                else
                {
                    str = str.Substring(i + 1);
                }//more to go
            }//length not 0
            return s;
        }//parse

        public static byte CheckSum(ref byte[] bb)
        {
            int j = 0;
            for (int i = 0; i < 7; i++) j = j + bb[i];
            return (byte)j;
        }//checksum

        public static void FillComboBoxWithPorts(ref ComboBox cb)
        {
            cb.Items.Clear();                  //a combo box
            cb.Sorted = true;
            string[] ports = SerialPort.GetPortNames();
            foreach (string port in ports)
            {
                cb.Items.Add(port);
            }//getting ports
            cb.SelectedIndex = 0;
        }//set comports

        // This routine uses a setup string like the one used in VB, etc.
        // Example: COM1,9600,8,1,N,1000
        // Explanation: ComPort, baudrate, databits, stopbits, parity, readtimeout
        // Use EndComm() to close it
        public static Boolean StartCom(ref SerialPort sp, String cPort, String setup)
        {
            int status = 0;
            string s1 = "";
            string s = setup;
            //configuring the serial port
            try
            {
                sp.PortName = cPort;
                sp.BaudRate = Int32.Parse(Parse(ref s));
                sp.DataBits = Int32.Parse(Parse(ref s));
                sp.StopBits = StopBits.One;
                s1 = Parse(ref s);
                if (s1 == "1.5") sp.StopBits = StopBits.OnePointFive;
                if (s1 == "2") sp.StopBits = StopBits.Two;
                sp.Parity = Parity.None;
                s1 = Parse(ref s);
                if (s1 == "E") sp.Parity = Parity.Even;
                if (s1 == "M") sp.Parity = Parity.Mark;
                if (s1 == "O") sp.Parity = Parity.Odd;
                if (s1 == "S") sp.Parity = Parity.Space;
                sp.ReadTimeout = Int32.Parse(Parse(ref s));
               
                //opening the serial port
                sp.Open();
                sp.DiscardOutBuffer();
                sp.DiscardInBuffer();
                status = 1;
            }//try to open
            catch
            {
                status = 0;
            }//catch
            if (status == 0) EndCom(ref sp);
            return (status == 1);
        }//open any comport

        // This routine always starts up a comport for the series 3000 controllers
        // Uses even parity etc. and isn't very standard
        public static Boolean StartCom(ref SerialPort sp, String cPort)
        {
            return StartCom(ref sp, cPort, "9600,8,1,E,5000");
        }//start comport

        public static Boolean EndCom(ref SerialPort sp)
        {
            if (sp.IsOpen)
            {
                sp.Close();
            }
            return (sp.IsOpen);
        }//stop comport

        // Use this for debugging to show the 8 bytes sent or received
        public static string ShowComm(byte[] ba)
        {
            string s = "";
            int i = 0;
            while (i < 8)
            {
                s = s + ba[i];
                if (i < 7) s = s + " ";
                i++;
            }//while
            return s;
        }//showcomm

        public static string GetData(ref SerialPort sp, string cmd)
        {
            string str = cmd;
            if (str.Length > 0)
            {
                string op = VtmComm.Parse(ref str);               //get operation	
                op = op.ToUpper();		                          //make upper case
                int sorc = ushort.Parse(VtmComm.Parse(ref str));  //get source
                int reg = ushort.Parse(VtmComm.Parse(ref str));   //get register		
                byte b = 0;                                       //b has op code
                short data = 0;                                   //data has write value
                if (op == "WRITE")
                {                                                 //set up write
                    b = 1;
                    data = short.Parse(VtmComm.Parse(ref str));
                }//if write	

                string s = "SErrSend";
                try
                {
                    //write data to serial port
                    byte[] ba = new byte[8] { 0x55, 0, 1, 0, 0, 0, 0, 0 };
                    ba[1] = (byte)sorc;
                    ba[4] = (byte)reg;
                    ba[3] = b;
                    ba[5] = (byte)((data & 0xFF00) >> 8);
                    ba[6] = (byte)data;
                    ba[7] = VtmComm.CheckSum(ref ba);

                    //write the data out
                    sp.Write(ba, 0, 8);
                    s = "TErrRecieve";

                    //read the data and store it
                    for (int i = 0; i < 8; i++) ba[i] = (Byte)sp.ReadByte();
                    byte bc = VtmComm.CheckSum(ref ba);
                    
                    s = "GErrUChar";
                    if (ba[0] == 0x55)
                    {
                        s = "GErrToken";
                        if (ba[3] == 2)
                        {
                            s = "GErrRegister";
                            if (ba[4] == (byte)reg)
                            {
                                s = "GErrSum";
                                if (bc == ba[7])
                                {
                                    data = (short)(((ba[5] << 8) & 0xFF00) | ba[6]);
                                    s = data.ToString();
                                }//start char ok
                            }//register ok
                        }//reply token ok
                    }//checksum ok
                    str = s;
                    return str;
                }//try

                catch
                {
                    str = s;                            //general error 

                }//catch                                   

            }//if
            return str;
        }//get data

        public static string GetData(ref SerialPort sp, int command, int source, int register, int value)
        {
            string str = "";
            byte b = 0;                                       //b has command code
            short data = 0;                                   //data has write value
            if (command == 1)
            {                                                 //set up write
                b = 1;
                data = (short)value;
            }//if write	
            try
            {
                //write data to serial port
                byte[] ba = new byte[] { 0x55, 0, 1, 0, 0, 0, 0, 0 };
                ba[1] = (byte)source;
                ba[4] = (byte)register;
                ba[3] = b;
                ba[5] = (byte)((data & 0xFF00) >> 8);
                ba[6] = (byte)data;
                ba[7] = CheckSum(ref ba);
                string s = "SErr";
                sp.Write(ba, 0, ba.Length);
                //read the data and store it
                for (int i = 0; i < 8; i++) ba[i] = (Byte)sp.ReadByte();
                s = "TErr";	     	                  //transmission error
                byte bc = CheckSum(ref ba);
                if (bc == ba[7])
                {
                    if (ba[3] == 2)
                    {
                        if (ba[4] == (byte)register)
                        {
                            if (ba[0] == 0x55)
                            {
                                data = (short)(((ba[5] << 8) & 0xFF00) | ba[6]);
                                s = data.ToString();
                            }//start char ok
                        }//register ok
                    }//reply token ok
                }//checksum ok
                str = s;
                return str;
            }//try
            catch
            {
                str = "GErr";                    //general error 
            }//catch                                   
            return str;
        }//get data 2

        public static bool GetThree(ref SerialPort sp, ref string cmd)
        {
            int i = 0;
            string str = "";
            bool b = false;
            while ((i < 3) & (b == false))
            {
                str = GetData(ref sp, cmd);
                b = false;
                if (str.IndexOf(@"Err") == 1) b = true;
                i++;
                if (b == true) Thread.Sleep(20);
                b = !b;
            }//while
            cmd = str;
            return b;
        }//get three

        public static bool GetThree(ref SerialPort sp, ref string cmd,
                                    int command, int source, int register, int value)
        {
            int i = 0;
            string str = "";
            bool b = false;
            while ((i < 3) & (b == false))
            {
                str = "";
                str = GetData(ref sp, command, source, register, value);
                if ((str == "GErr") | (str == "TErr") | (str == "SErr")) b = true;
                i++;
                if (b == true) Thread.Sleep(20);
                b = !b;
            }//while
            cmd = str;
            return b;
        }//get three 2

        public static bool ChangeSerialPort(ref SerialPort sp, string newPort)
        {
            EndCom(ref sp);
            return StartCom(ref sp, newPort);
        }//ChangeSerialPort

        public static bool Send(ref SerialPort sp, string str)
        {
            int status = 0;
            byte[] b = Encoding.ASCII.GetBytes(str);
            try
            {
                sp.Write(b,0,str.Length);
            }
            catch
            {
                status = 1;
            }
            return (status == 0);
        }//send a string

        public static string SendLine(ref SerialPort sp, string str)
        {
            string s = str + '\r' + '\n';
            try
            {
                sp.Write(s);
                s = sp.ReadLine();
            }
            catch
            {
                s = "GErr";
            }
            return s;
        }//send a string and read a line

        public static string Read(ref SerialPort sp)
        {
            string s = "";
            try
            {
                s = sp.ReadLine();
            }
            catch
            {
                s = "GErr";
            }
            return s;
        }//read a line from comport

        public static bool ReadBinary(ref SerialPort sp, ref byte[] ba, int num)
        {
            int status = 0;
            try
            {
                sp.Read(ba, 0, num);
            }
            catch
            {
                status = 1;
            }
            return (status == 0);
        }//read a line from comport






    }//vtmcomm
}//namespace
